@@ -6,7 +6,7 @@
     License as published by the Free Software Foundation; either
     version 2.1 of the License, or (at your option) version 3, or any
     later version accepted by the membership of KDE e.V. (or its
-    successor approved by the membership of KDE e.V.), Trolltech ASA 
+    successor approved by the membership of KDE e.V.), Nokia Corporation 
     (or its successors, if any) and the KDE Free Qt Foundation, which shall
     act as a proxy defined in Section 6 of version 3 of the license.
 
@@ -176,8 +176,8 @@ FactoryPrivate::FactoryPrivate()
     // as the whole backend might still be alive.
     qAddPostRoutine(globalFactory.destroy);
 #ifndef QT_NO_DBUS
-    QDBusConnection::sessionBus().connect(QString(), QString(), "org.kde.Phonon.Factory",
-            "phononBackendChanged", this, SLOT(phononBackendChanged()));
+    QDBusConnection::sessionBus().connect(QString(), QString(), QLatin1String("org.kde.Phonon.Factory"),
+        QLatin1String("phononBackendChanged"), this, SLOT(phononBackendChanged()));
 #endif
 }
 
@@ -325,6 +325,7 @@ FACTORY_IMPL(AudioOutput)
 #ifndef QT_NO_PHONON_VIDEO
 FACTORY_IMPL(VideoWidget)
 #endif //QT_NO_PHONON_VIDEO
+FACTORY_IMPL(AudioDataOutput)
 
 #undef FACTORY_IMPL
 
