@@ -21,6 +21,7 @@
 
 #include "backend.h"
 #include <phonon/experimental/backendinterface.h>
+#include <phonon/pulsesupport_p.h>
 #include "mediaobject.h"
 #include "effect.h"
 #include "events.h"
@@ -71,6 +72,10 @@ Backend::Backend(QObject *parent, const QVariantList &)
     m_debugMessages(!qgetenv("PHONON_XINE_DEBUG").isEmpty()),
     m_thread(0)
 {
+    // Initialise PulseAudio support
+    PulseSupport *pulse = PulseSupport::getInstance();
+    connect(pulse, SIGNAL(objectDescriptionChanged(ObjectDescriptionType)), SLOT(emitObjectDescriptionChanged(ObjectDescriptionType)));
+
     Q_ASSERT(s_instance == 0);
     s_instance = this;
 
@@ -91,7 +96,7 @@ Backend::Backend(QObject *parent, const QVariantList &)
     m_deinterlaceMethod = cg.value("Settings/deinterlaceMethod", 0).toInt();
 
     signalTimer.setSingleShot(true);
-    connect(&signalTimer, SIGNAL(timeout()), SLOT(emitAudioDeviceChange()));
+    connect(&signalTimer, SIGNAL(timeout()), SLOT(emitAudioOutputDeviceChange()));
     QDBusConnection::sessionBus().registerObject("/internal/PhononXine", this, QDBusConnection::ExportScriptableSlots);
 
     debug() << Q_FUNC_INFO << "Using Xine version " << xine_get_version_string();
@@ -116,6 +121,7 @@ Backend::~Backend()
     }
 
     s_instance = 0;
+    PulseSupport::shutdown();
 }
 
 XineEngine Backend::xineEngineForStream()
@@ -205,6 +211,10 @@ QStringList Backend::availableMimeTypes() const
 
 QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
 {
+    PulseSupport *pulse = PulseSupport::getInstance();
+    if (pulse->isActive() && (Phonon::AudioOutputDeviceType == type || Phonon::AudioCaptureDeviceType == type))
+        return pulse->objectDescriptionIndexes(type);
+
     QList<int> list;
     switch(type)
     {
@@ -257,6 +267,10 @@ QList<int> Backend::objectDescriptionIndexes(ObjectDescriptionType type) const
 
 QHash<QByteArray, QVariant> Backend::objectDescriptionProperties(ObjectDescriptionType type, int index) const
 {
+    PulseSupport *pulse = PulseSupport::getInstance();
+    if (pulse->isActive() && (Phonon::AudioOutputDeviceType == type || Phonon::AudioCaptureDeviceType == type))
+        return pulse->objectDescriptionProperties(type, index);
+
     //debug() << Q_FUNC_INFO << type << index;
     QHash<QByteArray, QVariant> ret;
     switch (type) {
@@ -489,10 +503,15 @@ bool Backend::endConnectionChange(QSet<QObject *> nodes)
     return true;
 }
 
-void Backend::emitAudioDeviceChange()
+void Backend::emitAudioOutputDeviceChange()
 {
     debug() << Q_FUNC_INFO;
-    emit objectDescriptionChanged(AudioOutputDeviceType);
+    emitObjectDescriptionChanged(AudioOutputDeviceType);
+}
+
+void Backend::emitObjectDescriptionChanged(ObjectDescriptionType type)
+{
+    emit objectDescriptionChanged(type);
 }
 
 bool Backend::deinterlaceDVD()
@@ -547,9 +566,10 @@ QHash<QByteArray, QVariant> Backend::audioOutputProperties(int audioDevice)
             ret.insert("description", that->m_audioOutputInfos[i].description);
 
             const QString iconName = that->m_audioOutputInfos[i].icon;
-            if (!iconName.isEmpty()) {
+            if (!iconName.isEmpty())
+                ret.insert("icon", iconName);
+            else
                 ret.insert("icon", QLatin1String("audio-card"));
-            }
             ret.insert("available", that->m_audioOutputInfos[i].available);
 
             ret.insert("initialPreference", that->m_audioOutputInfos[i].initialPreference);
@@ -571,6 +591,9 @@ QHash<QByteArray, QVariant> Backend::audioOutputProperties(int audioDevice)
 
 QByteArray Backend::audioDriverFor(int audioDevice)
 {
+    if (PulseSupport::getInstance()->isActive())
+        return "pulseaudio";
+
     instance()->checkAudioOutputs();
     const Backend *const that = instance();
     for (int i = 0; i < that->m_audioOutputInfos.size(); ++i) {
@@ -609,6 +632,20 @@ void Backend::checkAudioOutputs()
 
         // This will list the audio drivers, not the actual devices.
         const char *const *outputPlugins = xine_list_audio_output_plugins(m_xine);
+
+        PulseSupport *pulse = PulseSupport::getInstance();
+        if (pulse->isActive()) {
+            for (int i = 0; outputPlugins[i]; ++i) {
+                if (0 == strcmp(outputPlugins[i], "pulseaudio")) {
+                    // We've detected the pulseaudio output plugin. We're done.
+                    return;
+                }
+            }
+
+            // We cannot find the output plugin, so let the support class know.
+            pulse->disable();
+        }
+
         for (int i = 0; outputPlugins[i]; ++i) {
             debug() << Q_FUNC_INFO << "outputPlugin: " << outputPlugins[i];
             if (0 == strcmp(outputPlugins[i], "alsa")) {
@@ -645,8 +682,8 @@ void Backend::checkAudioOutputs()
                             /*icon name */"audio-backend-jack", outputPlugins[i]);
             } else if (0 == strcmp(outputPlugins[i], "arts")) {
                 addAudioOutput(nextIndex++, -100, tr("aRts"),
-                        tr("<html><p>aRts is the old soundserver and media framework that was used "
-                            "in KDE2 and KDE3. Its use is discuraged.</p></html>"),
+                        tr("<html><p>aRts is the old sound server and media framework that was used "
+                            "in KDE2 and KDE3. Its use is discouraged.</p></html>"),
                         /*icon name */"audio-backend-arts", outputPlugins[i]);
             } else if (0 == strcmp(outputPlugins[i], "pulseaudio")) {
                 addAudioOutput(nextIndex++, 10, tr("PulseAudio"),
