@@ -344,12 +344,11 @@ QImage VideoWidget::snapshot() const
 {
     QImage img;
     QMutexLocker lock(&m_snapshotLock);
-    const_cast<VideoWidget *>(this)->upstreamEvent(new Event(Event::RequestSnapshot));
+    const_cast<VideoWidget *>(this)->upstreamEvent(new RequestSnapshotEvent(img, &m_snapshotWait) );
     if (m_snapshotWait.wait(&m_snapshotLock, 1000)) {
-        img = m_snapshotImage;
-        m_snapshotImage = QImage();
+      return img;
     }
-    return img;
+    return QImage();
 }
 
 /*
@@ -472,10 +471,6 @@ void VideoWidget::mouseMoveEvent(QMouseEvent *mev)
 {
     K_XT(VideoWidget);
 
-    if (cursor().shape() == Qt::BlankCursor) {
-        setCursor(QCursor(Qt::ArrowCursor));
-    }
-
     x11_rectangle_t   rect;
     xine_event_t      *event = new xine_event_t;
     xine_input_data_t *input = new xine_input_data_t;
@@ -643,12 +638,6 @@ void VideoWidget::downstreamEvent(Event *e)
             }
         }
         break;
-    case Event::SnapshotReady:
-        m_snapshotLock.lock();
-        m_snapshotImage = static_cast<const SnapshotReadyEvent *>(e)->image;
-        m_snapshotWait.wakeAll();
-        m_snapshotLock.unlock();
-        break;
     default:
         QCoreApplication::sendEvent(this, e);
         break;
