@@ -618,12 +618,6 @@ void XineStream::useGapOf(int gap)
     QCoreApplication::postEvent(this, new QEVENT(TransitionTypeChanged));
 }
 
-// called from main thread
-void XineStream::gaplessSwitchTo(const QUrl &url)
-{
-    gaplessSwitchTo(url.toEncoded());
-}
-
 // called from main thread
 void XineStream::gaplessSwitchTo(const QByteArray &mrl)
 {
@@ -1211,7 +1205,10 @@ bool XineStream::event(QEvent *ev)
                 default:
                     return true;
                 }
-                handleDownstreamEvent(new SnapshotReadyEvent(qimg));
+                   RequestSnapshotEvent *event=static_cast<RequestSnapshotEvent*>(ev);
+                   event->image=qimg;
+                   event->waitCondition->wakeAll();
+//                 handleDownstreamEvent(new SnapshotReadyEvent(qimg));
             }
         }
         return true;
@@ -1706,12 +1703,6 @@ xine_post_out_t *XineStream::videoOutputPort() const
     return xine_get_video_source(m_stream);
 }
 
-// called from main thread
-void XineStream::setUrl(const QUrl &url)
-{
-    setMrl(url.toEncoded());
-}
-
 // called from main thread
 void XineStream::setMrl(const QByteArray &mrl, StateForNewMrl sfnm)
 {
